--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 


--			TODO LIST:
-- Internal:
--		Cleanups
--				shorten variables a lot, remove the str/i crap
--				is it possible to make one format function that gets quest id's etc, and goes through everything?
--		Settings : Rename Tracked/Not Tracked to Visible/Hidden, easier to understand. - do this in code as well
--      <Industrial> tip:  align the number of items needed and total for each point to the right

-- Graphical:
--		On indent multiline quest texts (for example goto) should the whole text be indented or just the start?

-- Overriding blizzard:
--		On open blizzard questlog frame, warn about not using the old questlog.
--		When track quest using blizzard quest tracking function, warn about using builtin quest tracking.

-- Todo modules:
--			Quest statistics, on tooltip show % chance of a mob dropping quest item (store this db in memory, remove when quest finished).

nQuestLog = AceLibrary("AceAddon-2.0"):new(
	"AceHook-2.1", 
    "AceConsole-2.0", 
    "AceDB-2.0", 
    "AceEvent-2.0",
    "AceModuleCore-2.0",
    "FuBarPlugin-2.0")
    
local quixote = AceLibrary("Quixote-1.0")
local crayon = AceLibrary("Crayon-2.0")

local L = AceLibrary("AceLocale-2.2"):new("nQuestLog-Core")
L:RegisterTranslations("enUS", function() return {
	["Watched Quests"] = true,
	["Listing all quests"] = true,

	["Show/Hide objectives"] = true,
	["Listing all watched quests"] = true,

	["Default : "] = true,
	["Default quest objective tracking"] = true,

	["Reset manually watched objectives"] = true,
	["Reset manually watched objectives"] = true,

	["Hide completed"] = true,
	["Hidden"] = true,
	["Shown"] = true,
	["Smart"] = true,
	["Zone "] = true,

	[" hidden)"] = true,
	[" visible)"] = true,
} end)    

L:RegisterTranslations("koKR", function() return {
	["Watched Quests"] = "퀘스트 보기",
	["Listing all quests"] = "모든 퀘스트 리스트 보기",

	["Show/Hide objectives"] = "퀘스트 목표 보기/숨기기",
	["Listing all watched quests"] = "모든 퀘스트 리스트의 보기/숨기기를 바꿉니다.",

	["Default : "] = "기본값: ",
	["Default quest objective tracking"] = "퀘스트 목표 기록을 기본값으로 돌립니다.",

	["Reset manually watched objectives"] = "퀘스트 목표 보기 초기화",
	["Reset manually watched objectives"] = "선택한 퀘스트 목표 보기를 초기화 합니다",

	["Hide completed"] = "숨기기 완료",
	["Hidden"] = "숨김",
	["Shown"] = "보임",
	["Smart"] = "자동",
	["Zone "] = "지역",

	[" hidden)"] = "개 숨김)",
	[" visible)"] = "개 보임)",
} end)    

nQuestLog.defaultPosition = "LEFT"
nQuestLog.hasIcon = true
nQuestLog.defaultMinimapPosition = 250
nQuestLog.independentProfile = true

function nQuestLog:ToggleVisible(open)
	if (open) then
		self.db.char["visible"] = true
		nQuestLogFrame:SetWidth(self.db.profile.width)
		self:SetPosition()
		self:RefreshTracker()
	else
		self.db.char["visible"] = false
		nQuestLogFrame:Hide()
	end
end

function nQuestLog:RefreshMenu()
	local lastPrintedZone = {}
	local menuOrder = 10
	
	self.mainMenu.args["Quests"] = {
		type = 'group',
		name = L["Watched Quests"],
		order = 80,
		desc = L["Listing all quests"],
		args = {}
	}
	
	local strTrackingDefault = L["Hide completed"]
	if (self.db.profile.hideAllObjectives) then
		strTrackingDefault = L["Hidden"]
	elseif (not self.db.profile.hideCompletedObjectives) then
		strTrackingDefault = L["Shown"]
	end
	
	if (self.db.profile.smartQuestTracking) then
		strTrackingDefault = L["Smart"]
	end
	
	self.mainMenu.args["Objectives"] = {
		type = 'group',
		name =L["Show/Hide objectives"],
		order = 85,
		desc=L["Listing all watched quests"],
		args = {
			headerTop = {
				type = 'header',
				name = L["Default : "] .. strTrackingDefault,
				order = 1,
				desc = L["Default quest objective tracking"],
			},
			reset = {
				type = 'execute',
				name = L["Reset manually watched objectives"],
				order = 2,
				desc = L["Reset manually watched objectives"],
				func = function()
					self.db.char.watchedObjectives = {}
					nQuestLog:Refresh()
				end,
			}
		}
	}
	
	local function AddToMenu(data,strTitle,strFancyTitle,strZone,strDescription)
		local strMenuName = data.Menu
		if (lastPrintedZone[strMenuName] ~= strZone) then
			self.mainMenu.args[strMenuName].args[strZone] = {
				type = 'toggle',
				name = self:Format_QuestZone(strZone),
				order = menuOrder,
				desc = L["Zone "] .. strZone,
				get = function()
					if (self.db.char.hiddenZones[strZone]) then
						return false
					end
					return true
				end,
				set = function(v)
					if (not v) then
						self.db.char.hiddenZones[strZone] = true
					else
						self.db.char.hiddenZones[strZone] = nil
					end
					self:Refresh()
				end
			}
			lastPrintedZone[strMenuName] = strZone
			menuOrder = menuOrder + 1
		end


		self.mainMenu.args[strMenuName].args[strTitle] = {
	            type = 'toggle',
	            name = strFancyTitle,
	            desc = strDescription,
	            order = menuOrder,
	            get = data.get,
	            set = data.set,
		}
	end
	
	
	
	
	-- Scan all quests
	for _, tmp_Zone in quixote:IterZones() do
		for _,iQuestId in quixote:QuestsByZone(tmp_Zone) do
			-- TODO, clear up the variables we dont use and remove them.
			local strTitle, iLevel, strTag, iGroup, iComplete, iObjectives, strZone = quixote:GetQuestById(iQuestId)
			local strObjective, strDescription = quixote:GetQuestText(iQuestId) -- TODO make this use same function as ShowTooltip or something like that?  It'd be much prettier.
					
			-- Scan the objectives to get numbers of quest items we want:
			local iTotalGot, iTotalNeeded = 0,0
			for i=1,iObjectives do
				local _, strType, iGot, iNeeded,isDone = quixote:GetQuestStatusById(iQuestId, i)
				if (strType == "reputation") then iNeeded = 1	iGot = 0	if (isDone) then iGot = 1 end end
				iTotalGot = iTotalGot + iGot
				iTotalNeeded = iTotalNeeded + iNeeded
			end
			
			-- Then the quest:
			local strFancyTitle = self:Format_QuestTitle(strTitle,iLevel,strTag,iTotalGot,iTotalNeeded,iComplete)
			AddToMenu({Menu="Quests",
					get = function()
						if (self.db.char.watchedQuests[strTitle]) then
							return true
						end
						return false
		            end,
		            set = function(v)
		            	if (v) then
		            		self.db.char.watchedQuests[strTitle] = true
		            	else
		            		self.db.char.watchedQuests[strTitle] = nil
		            	end
		            	nQuestLog:Refresh()
		            end,},strTitle,strFancyTitle,strZone,strDescription)

			
			if (self.db.char.watchedQuests[strTitle]) then
				AddToMenu({Menu="Objectives",
			            get = function()
			            	return nQuestLog:ShouldShowObjective(strTitle,iTotalGot,iTotalNeeded)
			            end,
			            set = function(v)
			            	if (v) then
			            		self.db.char.watchedObjectives[strTitle] = true
			            	else
			            		self.db.char.watchedObjectives[strTitle] = false
			            	end
			            	nQuestLog:Refresh()
			            end,
				},strTitle,strFancyTitle,strZone,strDescription)
			end
			
			menuOrder = menuOrder + 1
		end
	end
end

function nQuestLog:SetPosition()
	nQuestLogFrame:ClearAllPoints()
	if (not self.db.profile["trackerGrowUp"]) then
		-- Grown down
		nQuestLogFrame:SetPoint("TOPLEFT",UIParent, "BOTTOMLEFT",self.db.profile["positionLeft"],self.db.profile["positionTop"])
	else
		-- Grow up
		nQuestLogFrame:SetPoint("BOTTOMLEFT",UIParent, "BOTTOMLEFT",self.db.profile["positionLeft"],self.db.profile["positionBottom"])
	end
end

function nQuestLog:SavePosition()
	self.db.profile["positionBottom"] = nQuestLogFrame:GetBottom()
	self.db.profile["positionLeft"] = nQuestLogFrame:GetLeft()
	self.db.profile["positionTop"] = nQuestLogFrame:GetTop()
end

function nQuestLog:RefreshTracker()
	-- Refreshes all tracker stuff
	
	-- Local functions
	function RefreshBackdrop()
		nQuestLogFrame:SetBackdrop({
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
				tile=true, 
				tileSize=16,
				edgeSize=16,
			insets = { left=5, right=5, top=5, bottom=5}
		})
		local cBorderColor = self:GetColor("trackerBorderColor")
		local cBackgroundColor = self:GetColor("trackerBackgroundColor")
		nQuestLogFrame:SetBackdropBorderColor(cBorderColor.r / 255, cBorderColor.g / 255, cBorderColor.b / 255, cBorderColor.a)
		nQuestLogFrame:SetBackdropColor(cBackgroundColor.r / 255, cBackgroundColor.g / 255, cBackgroundColor.b / 255, cBackgroundColor.a)
	end

	RefreshBackdrop()
	nQuestHeaderText:SetText(self:Colorize("nQuestLog","trackerHeader"))
	self:Refresh()
end


function nQuestLog:Refresh()
	self.refresh = nil
	if (not self.db.char["visible"] or self.refreshHold) then
		return
	end
	
	-- Local variables
	local iFrameHeight = 0
	local strLastPrintedZone = ""
	local iQuestNumber = 1	-- Temporary, used to get frame data.
	local bLastFrameQuest = false -- This is used for padding, can be 1
	local strTitle, iLevel, strTag, strObjective, iComplete, iObjectives, strZone
	
	local function HideButtons()
		-- Hide all QuestInfo we're not showing:
		for i=iQuestNumber,MAX_QUESTS * 2,1 do
			local frame = getglobal("nQuestLogButton" .. i)
			frame:Hide()
		end
	end
	
	local function SetFrameText(strText,iQuestId,strZone)
		-- Shows + sets text on a new frame
		local frame = getglobal("nQuestLogButton" .. iQuestNumber)
		local frameText = getglobal("nQuestLogButton" .. iQuestNumber .. "Text")
		if (iQuestId == nil) then
			-- Calculate how many quests are visible/not visible for this zone.
			local iAllQuests = 0
			local iVisibleQuests = 0
			for _,iQuestId in quixote:QuestsByZone(strZone) do
				local strQuestTitle = quixote:GetQuestById(iQuestId)
				iAllQuests = iAllQuests + 1
				if (self.db.char.watchedQuests[strQuestTitle]) then
					iVisibleQuests = iVisibleQuests + 1
				end
			end
		
		
			if (self.db.char.hiddenZones[strZone]) then
				strText = "+ " .. strText
			else
				strText = "- " .. strText
			end

			if (self.db.profile.showZoneVisibleStatus) then
				if (self.db.char.hiddenZones[strZone]) then
					strText = strText .. " (" .. iAllQuests .. L[" hidden)"]
				else
					strText = strText .. " (" .. iVisibleQuests .. "/" .. iAllQuests .. L[" visible)"]
				end
			elseif(iVisibleQuests < iAllQuests and not self.db.char.hiddenZones[strZone]) then
				strText = strText .. " *"
			end
		end
		frameText:SetText(strText)
		
		-- Font size
		local fontName, fontHeight, fontFlags = frameText:GetFont()
		local fontSize = self.db.profile.fontSize
		if (iQuestId == nil) then
			fontSize = self.db.profile.fontSizeZone
		end
		frameText:SetFont(fontName,fontSize,fontFlags)
		
		-- Padding
		if (iQuestNumber > 1) then
			local parent = getglobal("nQuestLogButton" .. iQuestNumber - 1)
			
			local padding = self.db.profile.questPadding
			if (iQuestId ~= nil) then
				if (bLastFrameQuest) then
					padding = self.db.profile.questPadding
				else
					padding = 0
				end
				bLastFrameQuest = true
			else
				padding = self.db.profile.zonePadding
				bLastFrameQuest = false
			end
			
			frame:SetPoint("TOPLEFT",parent,"BOTTOMLEFT",0,-padding)
			iFrameHeight = iFrameHeight + padding
		end
		
		frameText:SetWidth(nQuestLogFrame:GetWidth() - 12) -- -10 = adjust for 5pixel border., and 2 for good measure (no seriously, it wraps wrong without it)
		frame:SetWidth(frameText:GetWidth())
		frame:SetHeight(frameText:GetHeight())
		iFrameHeight = iFrameHeight + frame:GetHeight()
		
		frame:Show()
		if ((not iQuestId and self:ShouldHighlightZone(strZone)) or (iQuestId and self:ShouldHighlight(strTitle))) then
			local color = self:GetColor("questHighlight")
			frame:SetBackdropColor(color.r / 255,color.g / 255,color.b / 255,color.a)
		else
			--self:Print("Off : " .. strText)
			frame:SetBackdropColor(0,0,0,0)
		end
		frame.strZone = strZone
		frame.iQuestId = iQuestId
		iQuestNumber = iQuestNumber + 1
	end
	

	local function TrackQuest(iQuestId)
		local strObjectives, iTotalGot, iTotalNeeded = self:Format_QuestObjectives(strTitle,iObjectives,strObjective,iQuestId)
		local strTitle = self:Format_QuestTitle(strTitle,iLevel,strTag,iTotalGot,iTotalNeeded,iComplete)
		SetFrameText(strTitle .. strObjectives,iQuestId,strZone)
	end
	
	
	
	-- Scan all quests
	for _, tmp_Zone in quixote:IterZones() do
		if (self.db.char.hiddenZones[tmp_Zone]) then
			-- Zone for this quest is hidden
			if (self.db.profile.showEmptyZones) then
				SetFrameText(self:Format_QuestZone(tmp_Zone),nil,tmp_Zone)
			else
				-- Zone is hidden and we should not show empty zones, lets see if we have any quests tracked here.
				for _,iQuestId in quixote:QuestsByZone(tmp_Zone) do
					strTitle = quixote:GetQuestById(iQuestId)
					if (self.db.char.watchedQuests[strTitle]) then
						-- Yes we do, lets show the zone
						SetFrameText(self:Format_QuestZone(tmp_Zone),nil,tmp_Zone)
						break
					end
				end -- for
			end -- if showEmptyZones
		else
			-- Zone for this quest is not hidden
			SetFrameText(self:Format_QuestZone(tmp_Zone),nil,tmp_Zone)
			for _,iQuestId in quixote:QuestsByZone(tmp_Zone) do
				-- TODO, clear up the variables we dont use and remove them.
				strTitle, iLevel, strTag, iGroup, iComplete, iObjectives, strZone = quixote:GetQuestById(iQuestId)
				strObjective = quixote:GetQuestText(iQuestId)
				if (self.db.char.watchedQuests[strTitle]) then
					-- OK, we found a quest, lets add it.
					TrackQuest(iQuestId)
				end
			end
		end
	end
	
	-- Hide all QuestInfo we're not showing:
	HideButtons()
	if (nQuestHeader:IsVisible()) then
		iFrameHeight = iFrameHeight + nQuestHeader:GetHeight()
	end
	
	nQuestLogFrame:SetHeight(iFrameHeight + 10)

	if (iFrameHeight == 0) then
		nQuestLogFrame:Hide()
	else	
		nQuestLogFrame:Show()
		self:SavePosition()
	end
end

function nQuestLog:OnInitialize()
	-- TODO this is bad!  And only for letting us remove the icon from the fubar line
	-- 	Should be using OnEnable and OnDisable correctly
	self:CreateFrames()

	self.refreshHold = true
	
	if (self.db.profile["positionBottom"] == -1 or self.db.profile["profilepositionLeft"] == -1 or self.db.profile["profilepositionTop"] == -1) then
		nQuestLogFrame:ClearAllPoints()
		nQuestLogFrame:SetPoint("CENTER",0,0)
		self:SavePosition()
	end
	
	--Register Quixote Events:
	self:RegisterEvent("Quixote_Quest_Gained", "OnQuest_Gained")
	self:RegisterEvent("Quixote_Quest_Lost", "OnQuest_Lost")
	self:RegisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
	self:RegisterEvent("Quixote_Update", "Refresh")
	self:RegisterEvent("Quixote_Ready", "OnQuestLog_Ready",true)	

	-- Used for checking if we should refresh:
	self:RegisterEvent("ZONE_CHANGED", "OnRefreshCheck")
	self:RegisterEvent("ZONE_CHANGED_INDOORS", "OnRefreshCheck")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnRefreshCheck")
	
	-- Initiate modules
	self:LoadModules()

	self:MakeSettingsMenu(self.mainMenu.args["settings"])
end


